try:
    import unittest2 as unittest
except ImportError:
    import unittest

import time
import os
import logging

from wia import Wia
from wia.error import WiaError, WiaValidationError, WiaUnauthorisedError, WiaForbiddenError, WiaNotFoundError

class InitTest(unittest.TestCase):
    def test_init_access_token(self):
        logging.info("Starting test_init_access_token")
        access_token = os.environ['device_secret_key']
        wia = Wia()
        wia.access_token = access_token
        whoami = wia.WhoAmI.retrieve()
        self.assertIsInstance(whoami, type(wia.WhoAmI))
        access_token = None
        logging.info("Finished test_init_access_token")

    # def test_stream_connect(self):
    #     logging.info("Starting test_stream_connect")
    #     access_token = os.environ['device_secret_key']
    #     wia = Wia()
    #     wia.Stream.connect()
    #     count = 0
    #     while count <= 10:
    #         time.sleep(0.5)
    #         count += 1
    #         if wia.Stream.connected:
    #             break
    #     self.assertTrue(wia.Stream.connected)
    #     access_token = None
    #     logging.info("Finished test_stream_connect")
    #
if __name__ == '__main__':
    unittest.main()



# try:
#     import unittest2 as unittest
# except ImportError:
#     import unittest
#
# import time
# import os
# import logging
#
# from wia import Wia
# from wia.error import WiaError, WiaValidationError, WiaUnauthorisedError, WiaForbiddenError, WiaNotFoundError
#
# #def on_connect():
#     #print("on_connect called in test")
#
# class StreamTest(unittest.TestCase):
#
#     @classmethod
#     def setUpClass(cls):
#         wia = Wia()
#         wia.access_token = os.environ['user_secret_key']
#
#     def test_stream_connect(self):
#         # wia.Stream.client.on_connect = on_connect
#         wia = Wia()
#         wia.Stream.connect()
#         count = 0
#         while count <= 10:
#             time.sleep(0.5)
#             count += 1
#             if wia.Stream.connected:
#                 break
#         self.assertTrue(wia.Stream.connected)
#
# if __name__ == '__main__':
#     unittest2.main()



try:
    import unittest2 as unittest
except ImportError:
    import unittest

import logging
import time
import os

from wia import Wia
from wia.error import WiaError, WiaValidationError, WiaUnauthorisedError, WiaForbiddenError, WiaNotFoundError

class LocationsTest(unittest.TestCase):
    def test_locations_publish_rest(self):
        logging.info("Started test_locations_publish_rest")
        wia = Wia()
        wia.access_token = os.environ['device_secret_key']
        location = wia.Location.publish(latitude=50, longitude=60)
        self.assertTrue(location.id is not None)
        wia.access_token = None
        logging.info("Finished test_locations_publish_rest")

    def test_location_publish_wrong_params(self):
        logging.info("Started test_location_publish_wrong_params")
        wia = Wia()
        wia.access_token = os.environ['device_secret_key']
        location = wia.Location.publish(name='fail')
        self.assertIsInstance(location, WiaError)
        wia.access_token = None
        logging.info("Finished test_location_publish_wrong_params")

if __name__ == '__main__':
    unittest2.main()



import time
import logging

from wia import Wia
from wia.rest_client import post, get, put, delete
from wia.error import WiaError, WiaValidationError, WiaUnauthorisedError, WiaForbiddenError, WiaNotFoundError

class WiaResource(object):
    @classmethod
    def is_success(self, response):
        if response.status_code == 200 or response.status_code == 201:
            return True
        else:
            return False

    @classmethod
    def error_response(self, response):
        if response.status_code == 400:
            return WiaValidationError(response)
        elif response.status_code == 401:
            return WiaUnauthorisedError(response)
        elif response.status_code == 403:
            return WiaForbiddenError(response)
        elif response.status_code == 404:
            return WiaNotFoundError(response)
        else:
            return WiaError(response)

class WiaResourceDelete(WiaResource):
    def __init__(self, **kwargs):
        self.id = (kwargs['id'] if 'id' in kwargs else None)
        self.deleted = (kwargs['deleted'] if 'deleted' in kwargs else None)

class Space(WiaResource):
    def __init__(self, **kwargs):
        self.id = (kwargs['id'] if 'id' in kwargs else None)
        self.name = (kwargs['name'] if 'name' in kwargs else None)
        self.createdAt = (kwargs['createdAt'] if 'createdAt' in kwargs else None)
        self.updatedAt = (kwargs['updatedAt'] if 'updatedAt' in kwargs else None)

    @classmethod
    def create(self, **kwargs):
        path = 'spaces'
        response = post(path, kwargs)
        if WiaResource.is_success(response):
            return Space(**response.json())
        else:
            return WiaResource.error_response(response)

    @classmethod
    def retrieve(self, id):
        path = 'spaces/' + id
        response = get(path)
        if WiaResource.is_success(response):
            return Space(**response.json())
        else:
            return WiaResource.error_response(response)

    @classmethod
    def update(self, **kwargs):
        path = 'spaces/' + kwargs['id']
        dictCopy = dict(kwargs)
        del dictCopy['id']
        response = put(path, dictCopy)
        if WiaResource.is_success(response):
            return Space(**response.json())
        else:
            return WiaResource.error_response(response)

    @classmethod
    def list(self, **kwargs):
        response = get('spaces', **kwargs)
        if WiaResource.is_success(response):
            responseJson = response.json()
            spaces = []
            for space in responseJson['spaces']:
                spaces.append(Space(**space))
            return {'spaces':spaces,'count': responseJson['count']}
        else:
            return WiaResource.error_response(response)

class Device(WiaResource):
    def __init__(self, **kwargs):
        self.id = (kwargs['id'] if 'id' in kwargs else None)
        self.name = (kwargs['name'] if 'name' in kwargs else None)
        self.events = (kwargs['events'] if 'events' in kwargs else None)
        self.location = (kwargs['location'] if 'location' in kwargs else None)
        self.createdAt = (kwargs['createdAt'] if 'createdAt' in kwargs else None)
        self.updatedAt = (kwargs['updatedAt'] if 'updatedAt' in kwargs else None)

    @classmethod
    def create(self, **kwargs):
        path = 'devices'
        response = post(path, kwargs)
        if WiaResource.is_success(response):
            return Device(**response.json())
        else:
            return WiaResource.error_response(response)

    @classmethod
    def retrieve(self, id):
        path = 'devices/' + id
        response = get(path)
        if WiaResource.is_success(response):
            return Device(**response.json())
        else:
            return WiaResource.error_response(response)

    @classmethod
    def update(self, **kwargs):
        path = 'devices/' + kwargs['id']
        dictCopy = dict(kwargs)
        del dictCopy['id']
        response = put(path, dictCopy)
        if WiaResource.is_success(response):
            return Device(**response.json())
        else:
            return WiaResource.error_response(response)

    @classmethod
    def delete(self, id):
        path = 'devices/' + id
        response = delete(path)
        if WiaResource.is_success(response):
            return WiaResourceDelete(**response.json())
        else:
            return WiaResource.error_response(response)

    @classmethod
    def list(self, **kwargs):
        response = get('devices', **kwargs)
        if WiaResource.is_success(response):
            responseJson = response.json()
            devices = []
            for device in responseJson['devices']:
                devices.append(Device(**device))
            return {'devices':devices,'count': responseJson['count']}
        else:
            return WiaResource.error_response(response)

class Event(WiaResource):
    def __init__(self, **kwargs):
        self.id = (kwargs['id'] if 'id' in kwargs else None)
        self.name = (kwargs['name'] if 'name' in kwargs else None)
        self.data = (kwargs['data'] if 'data' in kwargs else None)
        self.file = (kwargs['file'] if 'file' in kwargs else None)
        self.timestamp = (kwargs['timestamp'] if 'timestamp' in kwargs else None)

    @classmethod
    def publish(self, **kwargs):
        path = 'events'
        if not ('file' in kwargs) and Wia().Stream.connected and Wia().client_id is not None:
            topic = 'devices/' + Wia().client_id + '/' + path + '/' + kwargs['name']
            Wia().Stream.publish(topic=topic, **kwargs)
            return Event()
        else:
            response = post(path, kwargs)
            if WiaResource.is_success(response):
                return Event(**response.json())
            else:
                return WiaResource.error_response(response)

    @classmethod
    def subscribe(self, **kwargs):
        device=kwargs['device']
        topic='devices/' + device + '/events/'
        if 'name' in kwargs:
            topic += kwargs['name']
        else:
            topic += '+'
        Wia().Stream.subscribe(topic=topic, func=kwargs['func'])

    @classmethod
    def unsubscribe(self, **kwargs):
        device=kwargs['device']
        topic='devices/' + device + '/events/'
        if 'name' in kwargs:
            topic += kwargs['name']
        else:
            topic += '+'
        Wia().Stream.unsubscribe(topic=topic)

    @classmethod
    def list(self, **kwargs):
        response = get('events', **kwargs)
        if WiaResource.is_success(response):
            responseJson = response.json()
            events = []
            for event in responseJson['events']:
                events.append(Event(**event))
            return {'events':events,'count': responseJson['count']}
        else:
            return WiaResource.error_response(response)

class Location(WiaResource):
    def __init__(self, **kwargs):
        self.id = (kwargs['id'] if 'id' in kwargs else None)
        self.latitude = (kwargs['latitude'] if 'latitude' in kwargs else None)
        self.longitude = (kwargs['longitude'] if 'longitude' in kwargs else None)
        self.altitude = (kwargs['altitude'] if 'altitude' in kwargs else None)
        self.timestamp = (kwargs['timestamp'] if 'timestamp' in kwargs else None)
        self.recievedTimestamp = (kwargs['recievedTimestamp'] if 'recievedTimestamp' in kwargs else None)

    @classmethod
    def publish(self, **kwargs):
        path = 'locations'
        if Wia().Stream.connected and Wia().client_id is not None:
            topic = 'devices/' + Wia().client_id + '/' + path
            Wia().Stream.publish(topic=topic, **kwargs)
            return Location()
        else:
            response = post(path, kwargs)
            if WiaResource.is_success(response):
                return Location(**response.json())
            else:
                return WiaResource.error_response(response)

    @classmethod
    def subscribe(self, **kwargs):
        device = kwargs['device']
        topic = 'devices/' + device + '/locations'
        Wia().Stream.subscribe(topic=topic, func=kwargs['func'])

    @classmethod
    def unsubscribe(self, **kwargs):
        device = kwargs['device']
        topic = 'devices/' + device + '/locations'
        Wia().Stream.unsubscribe(topic=topic)

    @classmethod
    def list(self, **kwargs):
        response = get('locations', **kwargs)
        if WiaResource.is_success(response):
            responseJson = response.json()
            locations = []
            for location in responseJson['locations']:
                locations.append(Location(**location))
            return {'locations':locations,'count': responseJson['count']}
        else:
            return WiaResource.error_response(response)

class Log(WiaResource):
    def __init__(self, **kwargs):
        self.level = (kwargs['level'] if 'level' in kwargs else None)
        self.message = (kwargs['message'] if 'message' in kwargs else None)
        self.data = (kwargs['data'] if 'data' in kwargs else None)
        self.timestamp = (kwargs['timestamp'] if 'timestamp' in kwargs else None)

    @classmethod
    def publish(self, **kwargs):
        path = 'logs'
        if Wia().Stream.connected and Wia().client_id is not None:
            topic = 'devices/' + Wia().client_id + '/' + path + '/' + kwargs['level']
            Wia().Stream.publish(topic=topic, **kwargs)
            return Log()
        else:
            response = post(path, kwargs)
            if WiaResource.is_success(response):
                return Log(**response.json())
            else:
                return WiaResource.error_response(response)

    @classmethod
    def subscribe(self, **kwargs):
        device=kwargs['device']
        topic='devices/' + device + '/logs/'
        if 'level' in kwargs:
            topic += kwargs['level']
        else:
            topic += '+'
        Wia().Stream.subscribe(topic=topic, func=kwargs['func'])

    @classmethod
    def unsubscribe(self, **kwargs):
        device=kwargs['device']
        topic='devices/' + device + '/logs/'
        if 'level' in kwargs:
            topic += kwargs['level']
        else:
            topic += '+'
        Wia().Stream.unsubscribe(topic=topic)

    @classmethod
    def list(self, **kwargs):
        response = get('logs', **kwargs)
        if WiaResource.is_success(response):
            responseJson = response.json()
            logs = []
            for log in responseJson['logs']:
                logs.append(Log(**log))
            return {'logs':logs,'count': responseJson['count']}
        else:
            return WiaResource.error_response(response)

class AccessToken(WiaResource):
    def __init__(self, **kwargs):
        self.accessToken = (kwargs['accessToken'] if 'accessToken' in kwargs else None)
        self.refreshToken = (kwargs['refreshToken'] if 'refreshToken' in kwargs else None)
        self.tokenType = (kwargs['tokenType'] if 'tokenType' in kwargs else None)
        self.expiresIn = (kwargs['expiresIn'] if 'expiresIn' in kwargs else None)
        self.scope = (kwargs['scope'] if 'scope' in kwargs else None)

    @classmethod
    def create(self, **kwargs):
        response = post('auth/token', kwargs)
        if WiaResource.is_success(response):
            return AccessToken(**response.json())
        else:
            return WiaResource.error_response(response)

class WhoAmI(WiaResource):
    def __init__(self, **kwargs):
        self.id = (kwargs['id'] if 'id' in kwargs else None)
        self.contextData = (kwargs['contextData'] if 'contextData' in kwargs else None)
        self.scope = (kwargs['scope'] if 'scope' in kwargs else None)

    @classmethod
    def retrieve(self):
        response = get('whoami')
        if WiaResource.is_success(response):
            return WhoAmI(**response.json())
        else:
            return WiaResource.error_response(response)



#!/usr/bin/env python
from setuptools import setup

with open('LONG_DESCRIPTION.rst') as f:
    long_description = f.read()

exec(open("wia/version.py").read())

install_requires = [
    "requests>=2.18,<3.0",
    "paho-mqtt>=1.3"
]

setup(
    name='wia',
    version=VERSION,
    description='This client library is designed to support the Wia APIs',
    author='Wia',
    author_email='team@wia.io',
    maintainer='Conall Laverty',
    maintainer_email='team@wia.io',
    url='https://github.com/wiaio/wia-python-sdk',
    license='MIT',
    test_suite='wia.test.all',
    tests_require=['unittest2', 'mock'],
    packages=['wia', 'wia.test', 'wia.test.resources'],
    long_description=long_description,
    classifiers=[
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5'
    ],
    install_requires=install_requires,
)



import os
import logging

if 'wia_env' in os.environ:
    wia_env = os.environ['wia_env']
else:
    wia_env = 'production'

if wia_env == 'test':
    logging.basicConfig(level=logging.DEBUG)
else:
    logging.basicConfig(level=logging.INFO)

class _Singleton(type):
    """ A metaclass that creates a Singleton base class when called. """
    _instances = {}
    def __call__(cls, *args, **kwargs):
        if cls not in cls._instances:
            cls._instances[cls] = super(_Singleton, cls).__call__(*args, **kwargs)
        return cls._instances[cls]

class Wia(_Singleton('SingletonMeta', (object,), {})):
    def __init__(self):
        from wia.resource import (
            Device,
            Event,
            Location,
            Log,
            Space,
            WhoAmI
        )

        from wia.stream_client import (
            Stream
        )

        from wia.version import VERSION

        self.__access_token = None
        self.__app_key = None
        self.__auth_info = None
        self.__client_id = None
        self.__rest_config = {"protocol":'https',"host":'api.wia.io',"port":443,"basePath":'v1'}
        self.__stream_config = {"protocol":'mqtt',"host":'api.wia.io',"port":1883}

        self.__version = VERSION

        self.Device = Device()
        self.Event = Event()
        self.Location = Location()
        self.Log = Log()
        self.Space = Space()
        self.Stream = Stream()
        self.WhoAmI = WhoAmI()

    @property
    def access_token(self):
        return self.__access_token

    @access_token.setter
    def access_token(self, value):
        self.__access_token = value
        if self.__access_token is not None:
            self.__auth_info = self.WhoAmI.retrieve()
            if self.__auth_info is not None:
                self.__client_id = self.__auth_info.id
                logging.debug("Setting client_id as %s", self.__client_id)
        else:
            self.__auth_info = None
            self.__client_id = None
            logging.debug("Resetting client info.")

    @property
    def app_key(self):
        return self.__app_key

    @app_key.setter
    def app_key(self, value):
        self.__app_key = value

    @property
    def rest_config(self):
        return self.__rest_config

    @rest_config.setter
    def rest_config(self, value):
        self.__rest_config = value

    @property
    def stream_config(self):
        return self.__stream_config

    @stream_config.setter
    def stream_config(self, value):
        self.__stream_config = value

    @property
    def client_id(self):
        return self.__client_id

    @client_id.setter
    def client_id(self, value):
        self.__client_id = value



try:
    import unittest2 as unittest
except ImportError:
    import unittest

import logging
import time
import os

from wia import Wia
from wia.error import WiaError, WiaValidationError, WiaUnauthorisedError, WiaForbiddenError, WiaNotFoundError

class EventsTest(unittest.TestCase):
    def test_events_publish_rest(self):
        wia = Wia()
        wia.access_token = os.environ['device_secret_key']
        event = wia.Event.publish(name='test_event_other_rest', data=130)
        self.assertTrue(event.id is not None)
        wia.access_token = None

    # def test_events_publish(self):
    #     wia = Wia()
    #     wia.access_token = os.environ['device_secret_key']
    #     wia.Stream.connect()
    #     count = 0
    #     while count <= 10:
    #         time.sleep(0.5)
    #         count += 1
    #         if wia.Stream.connected:
    #             time.sleep(1)
    #             break
    #     self.assertTrue(wia.Stream.connected)
    #     result = wia.Event.publish(name='test_event_mqtt', data=130)
    #     self.assertTrue(result is not None)
    #     wia.Stream.disconnect()
    #     count = 0
    #     while count <= 10:
    #         time.sleep(0.5)
    #         count += 1
    #         if not wia.Stream.connected:
    #             break
    #     self.assertFalse(wia.Stream.connected)
    #     wia.access_token = None

    # def test_events_subscribe(self):
    #     wia = Wia()
    #     def wildcard_function(payload):
    #         pass
    #     def specific_function(payload):
    #         pass
    #     wia.access_token = os.environ['org_secret_key']
    #     wia.Stream.connect()
    #     count = 0
    #     # waits for Stream to be connected
    #     while count <= 10:
    #         time.sleep(0.5)
    #         count += 1
    #         if wia.Stream.connected:
    #             time.sleep(1)
    #             break
    #     self.assertTrue(wia.Stream.connected)
    #
    #     # subscirbe to event
    #     wia.Event.subscribe(device=os.environ['device_id'], func=wildcard_function)
    #     wia.Event.subscribe(device=os.environ['device_id'], name='subscribe_test_event', func=specific_function)
    #     count = 0
    #     while count <= 10:
    #         time.sleep(0.5)
    #         count += 1
    #         if wia.Stream.subscribed:
    #             time.sleep(1)
    #             break
    #     self.assertTrue(wia.Stream.subscribed)
    #
    #     wia.Stream.disconnect()
    #     count = 0
    #     while count <= 10:
    #         time.sleep(0.5)
    #         count += 1
    #         if not wia.Stream.connected:
    #             break
    #     self.assertFalse(wia.Stream.connected)
    #
    #     # publish event data
    #     wia.access_token = os.environ['device_secret_key']
    #
    #     wia.Stream.connect()
    #     count = 0
    #     while count <= 10:
    #         time.sleep(0.5)
    #         count += 1
    #         if wia.Stream.connected:
    #             break
    #     self.assertTrue(wia.Stream.connected)
    #
    #     wia.Event.publish(name='subscribe_test_event', data=97)
    #
    #     wia.access_token = os.environ['org_secret_key']
    #     time.sleep(0.5)
    #
    #     count = 0
    #     initial_subscribe_count = wia.Stream.subscribed_count
    #     #print(initial_subscribe_count)
    #
    #     # unsubscribe from event
    #     wia.Event.unsubscribe(device=os.environ['device_id'])
    #     wia.Event.unsubscribe(device=os.environ['device_id'], name='subscribe_test_event')
    #     while count <= 10:
    #         time.sleep(0.5)
    #         count += 1
    #         if wia.Stream.subscribed_count < initial_subscribe_count:
    #             break
    #     self.assertTrue(wia.Stream.subscribed_count < initial_subscribe_count)
    #     wia.Stream.disconnect()
    #     count = 0
    #     while count <= 10:
    #         time.sleep(0.5)
    #         count += 1
    #         if not wia.Stream.connected:
    #             break
    #     self.assertFalse(wia.Stream.connected)
    #     wia.access_token = None

    # ERROR TESTS
    def test_events_publish_rest_error(self):
        wia = Wia()
        wia.access_token = os.environ['device_secret_key']
        result = wia.Event.publish(abc='def')
        self.assertIsInstance(result, WiaError)
        wia.access_token = None

if __name__ == '__main__':
    unittest.main()



import requests
import logging

from wia import Wia

'''
wia_post:
    args:
        path:   string specifying url path
        kwargs: variable-length dict which can
                contain data for post request
'''
def post(path, kwargs):
    url = generate_url(path)
    headers = generate_headers()

    if 'file' in kwargs:
        logging.debug("Has file argument.")
        kwargsCopy = dict(kwargs)
        del kwargsCopy['file']
        r = requests.post(url, data=kwargsCopy, headers=headers, files={'file': kwargs['file']})
    else:
        logging.debug("No file argument. Posting as JSON.")
        r = requests.post(url, json=kwargs, headers=headers)
    return r

'''
wia_put:
    args:
        path:   string specifying url path
        kwargs: variable-length dict which can
                contain data for put request
'''
def put(path, kwargs):
    url = generate_url(path)
    headers = generate_headers()
    data = kwargs
    r = requests.put(url, json=data, headers=headers)
    return r

'''
wia_get:
    args:
        path:   string specifying url path
        kwargs: variable-length dict which can
                contain query params
'''
def get(path, **kwargs):
    url = generate_url(path)
    headers = generate_headers()

    r = requests.get(url, headers=headers, params=kwargs)
    return r

'''
wia_delete:
    args:
        path: string specifying url path
'''
def delete(path):
    url = generate_url(path)
    headers = generate_headers()

    r = requests.delete(url, headers=headers)
    return r

def generate_url(path):
    if path is None:
        path = ''

    url = Wia().rest_config['protocol'] + '://' + Wia().rest_config['host'] + '/' + Wia().rest_config['basePath'] + '/' + path

    logging.debug('URL: %s', url)

    return url

def generate_headers():
    headers = {}

    if Wia().access_token is not None:
        headers['Authorization'] = 'Bearer ' + Wia().access_token

    if Wia().app_key is not None:
        headers['x-app-key'] = Wia().app_key

    logging.debug('Headers: %s', headers)

    return headers



import sys

class WiaError(object):
    def __new__(cls, *p, **k):
        inst = object.__new__(cls)
        return inst

    def __init__(self, response):
        self.status_code = response.status_code
        self.json = response.json()
        self.headers = response.headers
        #self.message = message

# Status code: 400
class WiaValidationError(WiaError):
    pass

# Status code: 401
class WiaUnauthorisedError(WiaError):
    pass

# Status code: 403
class WiaForbiddenError(WiaError):
    pass

# Status code: 404
class WiaNotFoundError(WiaError):
    pass
